/*
 * tools.h: SWMOD deamon
 *
 * Copyright (C) 2020 iopsys Software Solutions AB. All rights reserved.
 *
 * Author: Amin Ben Ramdhane <amin.benramdhane@pivasoftware.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef __TOOLS_H
#define __TOOLS_H

#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <libubox/list.h>
#include <syslog.h>

#include "swmod.h"

#define SWMOD_PATH "/etc/swmod/"
#define OPKG_INFO_PATH "/usr/lib/opkg/info/"
#define SWMOD_MAP_DU "map_du"
#define STD_UCI_PATH "/etc/config/"
#define STD_INIT_PATH "/etc/init.d/"
#define LXC_AUTOBOOT_FILE "lxc-auto"

#define SYSFS_FOREACH_FILE(path, dir, ent) \
	if ((dir = opendir(path)) == NULL) return 0; \
	while ((ent = readdir (dir)) != NULL) \

#ifndef FREE
#define FREE(x) do { if(x) {free(x); x = NULL;} } while (0)
#endif

enum swmod_action_enum {
	SWMOD_DEPLOYMENT_UNIT,
	SWMOD_EXECUTION_UNIT
};

struct buffer_list
{
	struct list_head list;
	char line[MAX_LEN_128];
};

void create_buffer_list(struct list_head *opkg_list, char *buffer, int buff_len);
void buffer_add_line(struct list_head *head, char *entry);
void delete_buffer_list(struct list_head *opkg_list);
bool present_in_buffer_list(struct list_head *plist, char *entry);

bool file_exists(const char *path);
bool create_dir(const char *path);
int run_cmd(const char *cmd, char *output, size_t out_len);
void remove_new_line(char *buf);
void create_file(const char *path);
void swmod_strncpy(char *dst, const char *src, size_t n);
char *generate_uuid(void);
char *generate_duid(bool sysnchronise, int number);
int wait_for_pid(pid_t pid);

void configure_debug_level(unsigned char level);
void print_error(const char *format, ...);
void print_warning(const char *format, ...);
void print_info(const char *format, ...);
void print_debug(const char *format, ...);
int get_pid_details(int pid, char *cmdline, int cmd_len, int *vsize);
void get_ubus_output(char *object, char *method, ubus_data_handler_t callback, void *data);
void update_eu_from_blob(ExecEnv_t *env, struct blob_attr *msg);
int ubus_call_service_state(char *service, bool state);
bool dir_exist(const char *dir);
bool get_autoboot_from_config_file(const char *env);
void set_autoboot_to_config_file(const char *env);
void delete_autoboot_from_config_file(const char *env);
int get_timeout_from_config_file(const char *env);
void swmod_add_eu_in_list(struct list_head *eu_list, ExecUnit *eu);
void swmod_delete_eu_list(struct list_head *eu_list);

#define PRINT_DEBUG(fmt, args...) \
	print_debug("[%s:%d]"fmt, __func__, __LINE__, ##args)

#define PRINT_INFO(fmt, args...) \
	print_info("[%s:%d] " fmt, __func__, __LINE__, ##args)

#define PRINT_ERR(fmt, args...) \
	print_error("[%s:%d] " fmt, __func__, __LINE__, ##args)

#define PRINT_WARNING(fmt, args...) \
	print_warning("[%s:%d] " fmt, __func__, __LINE__, ##args)

#endif //__TOOLS_H

